// Code generated by bib. DO NOT EDIT.

package main

var bibliography = []struct{
    CiteName  string
    URL       string
    Formatted string
}{

    {
        CiteName:  "addchainreview",
        URL: "https://thescipub.com/pdf/10.3844/jcssp.2017.275.289.pdf",
        Formatted: "Adamu Muhammad Noma, Abdullah Muhammed, Mohamad Afendee Mohamed and Zuriati Ahmad Zulkarnain. A Review on Heuristics for Addition Chain Problem: Towards Efficient Public Key Cryptosystems. Journal of Computer Science. 2017. https://thescipub.com/pdf/10.3844/jcssp.2017.275.289.pdf",
    },

    {
        CiteName:  "aranha",
        URL: "https://eprint.iacr.org/2013/647",
        Formatted: "Diego F. Aranha, Paulo S. L. M. Barreto, Geovandro C. C. F. Pereira and Jefferson E. Ricardini. A note on high-security general-purpose elliptic curves. Cryptology ePrint Archive, Report 2013/647. 2013. https://eprint.iacr.org/2013/647",
    },

    {
        CiteName:  "boscoster",
        URL: "https://link.springer.com/content/pdf/10.1007/0-387-34805-0_37.pdf",
        Formatted: "Bos, Jurjen and Coster, Matthijs. Addition Chain Heuristics. In Advances in Cryptology --- CRYPTO' 89 Proceedings, pages 400--407. 1990. https://link.springer.com/content/pdf/10.1007/0-387-34805-0_37.pdf",
    },

    {
        CiteName:  "braueraddsubchains",
        URL: "http://www.martin-otto.de/publications/docs/2001_MartinOtto_Diplom_BrauerAddition-SubtractionChains.pdf",
        Formatted: "Martin Otto. Brauer addition-subtraction chains. PhD thesis, Universitat Paderborn. 2001. http://www.martin-otto.de/publications/docs/2001_MartinOtto_Diplom_BrauerAddition-SubtractionChains.pdf",
    },

    {
        CiteName:  "contfrac",
        URL: "http://www-igm.univ-mlv.fr/~berstel/Articles/1989AdditionChainDuboc.pdf",
        Formatted: "F Bergeron, J Berstel, S Brlek and C Duboc. Addition chains using continued fractions. Journal of Algorithms. 1989. http://www-igm.univ-mlv.fr/~berstel/Articles/1989AdditionChainDuboc.pdf",
    },

    {
        CiteName:  "crandallprime",
        URL: "https://patents.google.com/patent/US5159632A",
        Formatted: "Richard E. Crandall. Method and apparatus for public key exchange in a cryptographic system. US Patent 5,159,632. 1992. https://patents.google.com/patent/US5159632A",
    },

    {
        CiteName:  "curve25519",
        URL: "https://cr.yp.to/ecdh/curve25519-20060209.pdf",
        Formatted: "Bernstein, Daniel J. Curve25519: New Diffie-Hellman Speed Records. In Public Key Cryptography - PKC 2006, pages 207--228. 2006. https://cr.yp.to/ecdh/curve25519-20060209.pdf",
    },

    {
        CiteName:  "curvechains",
        URL: "https://briansmith.org/ecc-inversion-addition-chains-01",
        Formatted: "Brian Smith. The Most Efficient Known Addition Chains for Field Element and Scalar Inversion for the Most Popular and Most Unpopular Elliptic Curves. 2017. https://briansmith.org/ecc-inversion-addition-chains-01 (accessed June 30, 2019)",
    },

    {
        CiteName:  "efficientcompaddchain",
        URL: "http://www.numdam.org/item/JTNB_1994__6_1_21_0",
        Formatted: "Bergeron, F., Berstel, J. and Brlek, S. Efficient computation of addition chains. Journal de theorie des nombres de Bordeaux. 1994. http://www.numdam.org/item/JTNB_1994__6_1_21_0",
    },

    {
        CiteName:  "elligator",
        URL: "https://eprint.iacr.org/2013/325",
        Formatted: "Daniel J. Bernstein, Mike Hamburg, Anna Krasnova and Tanja Lange. Elligator: Elliptic-curve points indistinguishable from uniform random strings. Cryptology ePrint Archive, Report 2013/325. 2013. https://eprint.iacr.org/2013/325",
    },

    {
        CiteName:  "fips186-2",
        URL: "https://csrc.nist.gov/csrc/media/publications/fips/186/2/archive/2000-01-27/documents/fips186-2.pdf",
        Formatted: "NIST. Digital Signature Standard (DSS). Federal Information Processing Standards Publication 186-2. 2000. https://csrc.nist.gov/csrc/media/publications/fips/186/2/archive/2000-01-27/documents/fips186-2.pdf",
    },

    {
        CiteName:  "gencontfrac",
        URL: "https://eprint.iacr.org/2013/466",
        Formatted: "Amadou Tall and Ali Yassin Sanghare. Efficient computation of addition-subtraction chains using generalized continued Fractions. Cryptology ePrint Archive, Report 2013/466. 2013. https://eprint.iacr.org/2013/466",
    },

    {
        CiteName:  "genshortchains",
        URL: "https://pdfs.semanticscholar.org/b398/d10faca35af9ce5a6026458b251fd0a5640c.pdf",
        Formatted: "Kunihiro, Noboru and Yamamoto, Hirosuke. New Methods for Generating Short Addition Chains. IEICE Transactions on Fundamentals of Electronics Communications and Computer Sciences. 2000. https://pdfs.semanticscholar.org/b398/d10faca35af9ce5a6026458b251fd0a5640c.pdf",
    },

    {
        CiteName:  "github:kwantam/addchain",
        URL: "https://github.com/kwantam/addchain",
        Formatted: "Riad S. Wahby. kwantam/addchain. Github Repository. Apache License, Version 2.0. 2018. https://github.com/kwantam/addchain",
    },

    {
        CiteName:  "goldilocks",
        URL: "https://eprint.iacr.org/2015/625",
        Formatted: "Mike Hamburg. Ed448-Goldilocks, a new elliptic curve. Cryptology ePrint Archive, Report 2015/625. 2015. https://eprint.iacr.org/2015/625",
    },

    {
        CiteName:  "hashbls12-381",
        URL: "https://eprint.iacr.org/2019/403",
        Formatted: "Riad S. Wahby and Dan Boneh. Fast and simple constant-time hashing to the BLS12-381 elliptic curve. Cryptology ePrint Archive, Report 2019/403. 2019. https://eprint.iacr.org/2019/403",
    },

    {
        CiteName:  "hehcc:exp",
        URL: "http://koclab.cs.ucsb.edu/teaching/ecc/eccPapers/Doche-ch09.pdf",
        Formatted: "Christophe Doche. Exponentiation. Handbook of Elliptic and Hyperelliptic Curve Cryptography, chapter 9. 2006. http://koclab.cs.ucsb.edu/teaching/ecc/eccPapers/Doche-ch09.pdf",
    },

    {
        CiteName:  "knuth",
        URL: "https://www-cs-faculty.stanford.edu/~knuth/taocp.html",
        Formatted: "Knuth, Donald E. Evaluation of Powers. The Art of Computer Programming, Volume 2 (Third Edition): Seminumerical Algorithms, chapter 4.6.3. 1997. https://www-cs-faculty.stanford.edu/~knuth/taocp.html",
    },

    {
        CiteName:  "modboscoster",
        URL: "http://library.isical.ac.in:8080/jspui/bitstream/10263/6441/1/DISS-285.pdf",
        Formatted: "Ayan Nandy. Modifications of Bos and Coster’s Heuristics in search of a shorter addition chain for faster exponentiation. Masters thesis, Indian Statistical Institute Kolkata. 2011. http://library.isical.ac.in:8080/jspui/bitstream/10263/6441/1/DISS-285.pdf",
    },

    {
        CiteName:  "mpnt",
        URL: "https://profs.info.uaic.ro/~tr/tr03-02.pdf",
        Formatted: "F. L. Ţiplea, S. Iftene, C. Hriţcu, I. Goriac, R. Gordân and E. Erbiceanu. MpNT: A Multi-Precision Number Theory Package, Number Theoretical Algorithms (I). Technical Report TR03-02, Faculty of Computer Science, \"Alexandru Ioan Cuza\" University, Iasi. 2003. https://profs.info.uaic.ro/~tr/tr03-02.pdf",
    },

    {
        CiteName:  "nistdanger",
        URL: "https://cr.yp.to/talks/2013.09.16/slides-djb-20130916-a4.pdf",
        Formatted: "Daniel J. Bernstein and Tanja Lange. Security dangers of the NIST curves. 2013. https://cr.yp.to/talks/2013.09.16/slides-djb-20130916-a4.pdf",
    },

    {
        CiteName:  "pairingsfinalexp",
        URL: "https://eprint.iacr.org/2008/490",
        Formatted: "Michael Scott, Naomi Benger, Manuel Charlemagne, Luis J. Dominguez Perez and Ezekiel J. Kachisa. On the final exponentiation for calculating pairings on ordinary elliptic curves. Cryptology ePrint Archive, Report 2008/490. 2008. https://eprint.iacr.org/2008/490",
    },

    {
        CiteName:  "safecurves",
        URL: "https://safecurves.cr.yp.to",
        Formatted: "Daniel J. Bernstein and Tanja Lange. SafeCurves: choosing safe curves for elliptic-curve cryptography. https://safecurves.cr.yp.to",
    },

    {
        CiteName:  "sec2",
        URL: "https://safecurves.cr.yp.to/www.secg.org/sec2-v2.pdf",
        Formatted: "Certicom Research. SEC 2: Recommended Elliptic Curve Domain Parameters, Version 2.0. Standards for Efficient Cryptography 2. 2010. https://safecurves.cr.yp.to/www.secg.org/sec2-v2.pdf",
    },

    {
        CiteName:  "solinasprime",
        URL: "http://cacr.uwaterloo.ca/techreports/1999/corr99-39.pdf",
        Formatted: "Jerome A. Solinas. Generalized Mersenne Primes. Technical Report CORR 99-39, Centre for Applied Cryptographic Research (CACR) at the University of Waterloo. 1999. http://cacr.uwaterloo.ca/techreports/1999/corr99-39.pdf",
    },

    {
        CiteName:  "speedsubgroup",
        URL: "https://cr.yp.to/bib/2003/stam-thesis.pdf",
        Formatted: "Stam, Martijn. Speeding up subgroup cryptosystems. PhD thesis, Technische Universiteit Eindhoven. 2003. https://cr.yp.to/bib/2003/stam-thesis.pdf",
    },

}
